@@ -13,8 +13,7 @@ Created on 30 Sep 2020
 
 from time import time
 from datetime import datetime
-from pynmea2 import parse as nmea_parse, ParseError, types, RMC, VTG, GSV, GGA, GSA, GLL
-from pynmea2.types.proprietary.ubx import UBX00
+from pynmeagps import NMEAReader, NMEAMessage, NMEAParseError, VALCKSUM, GET
 
 from .globals import (
     DEVICE_ACCURACY,
@@ -22,14 +21,13 @@ from .globals import (
     SAT_EXPIRY,
     knots2ms,
     kmph2ms,
-    nmea2latlon,
 )
 from .strings import NMEAVALERROR
 
 
 class NMEAHandler:
     """
-    NMEAHandler class
+    NMEA handler class.
     """
 
     def __init__(self, app):
@@ -44,7 +42,6 @@ class NMEAHandler:
 
         self._raw_data = None
         self._parsed_data = None
-        self._record_track = False
         self.gsv_data = (
             []
         )  # Holds array of current satellites in view from NMEA GSV sentences
@@ -69,38 +66,42 @@ class NMEAHandler:
 
         :param bytes data: parsed NMEA sentence
         """
+        # pylint: disable=no-member
+
+        if data is None:
+            return None
 
         try:
-            parsed_data = nmea_parse(data)
-        except ParseError:
+            parsed_data = NMEAReader.parse(data, validate=VALCKSUM, msgmode=GET)
+        except NMEAParseError as err:
             # Parsing errors at this point are typically due to NMEA and UBX
             # protocols getting garbled in the input stream. It only happens
             # rarely so we ignore them and carry on.
+            # print(f"DEBUG NMEAHandler.process_data - err = {err}")
             return None
 
         if data or parsed_data:
             self._update_console(data, parsed_data)
-
-        if isinstance(parsed_data, RMC):  # Recommended minimum data for GPS
+        if parsed_data.msgID == "RMC":  # Recommended minimum data for GPS
             self._process_RMC(parsed_data)
-        if isinstance(parsed_data, GGA):  # GPS Fix Data
+        if parsed_data.msgID == "GGA":  # GPS Fix Data
             self._process_GGA(parsed_data)
-        if isinstance(parsed_data, GLL):  # GLL Lat Lon Data
+        if parsed_data.msgID == "GLL":  # GPS Lat Lon Data
             self._process_GLL(parsed_data)
-        if isinstance(
-            parsed_data, GSA
-        ):  # GPS DOP (Dilution of Precision) and active satellites
+        if parsed_data.msgID == "GSA":  # GPS DOP (Dilution of Precision)
             self._process_GSA(parsed_data)
-        if isinstance(parsed_data, VTG):  # GPS Vector track and Speed over the Ground
+        if parsed_data.msgID == "VTG":  # GPS Vector track and Speed over the Ground
             self._process_VTG(parsed_data)
-        if isinstance(parsed_data, GSV):  # GPS Satellites in View
+        if parsed_data.msgID == "GSV":  # GPS Satellites in View
             self._process_GSV(parsed_data)
-        if isinstance(parsed_data, UBX00):  # GPS Satellites in View
+        if (
+            parsed_data.msgID == "UBX" and parsed_data.msgId == 0
+        ):  # GPS Lat/Lon & Acc Data
             self._process_UBX00(parsed_data)
 
         return self._parsed_data
 
-    def _update_console(self, raw_data: bytes, parsed_data: types.talker):
+    def _update_console(self, raw_data: bytes, parsed_data):
         """
         Write the incoming data to the console in raw or parsed format.
 
@@ -108,24 +109,26 @@ class NMEAHandler:
         :param pynmea2.types.talker parsed_data: parsed data
         """
 
-        if self.__app.frm_settings.get_settings()["raw"]:
-            self.__app.frm_console.update_console(repr(raw_data))
+        if self.__app.frm_settings.raw:
+            self.__app.frm_console.update_console(str(raw_data).strip("\n"))
         else:
-            self.__app.frm_console.update_console(repr(parsed_data))
+            self.__app.frm_console.update_console(str(parsed_data))
 
-    def _process_RMC(self, data: types.talker):
+    def _process_RMC(self, data: NMEAMessage):
         """
         Process RMC sentence - Recommended minimum data for GPS.
 
-        :param pynmea2.types.talker data: parsed RMC sentence
+        :param pynmeagps.NMEAMessage data: parsed RMC sentence
         """
 
         try:
-            self.utc = data.timestamp
-            (self.lat, self.lon) = nmea2latlon(data)
-            if data.spd_over_grnd != "None":
-                self.speed = knots2ms(data.spd_over_grnd)  # convert to m/s
-            self.track = data.true_course
+            self.utc = data.time
+            self.lat = data.lat
+            self.lon = data.lon
+            if data.spd != "":
+                self.speed = knots2ms(data.spd)  # convert to m/s
+            if data.cog != "":
+                self.track = data.cog
             self.__app.frm_banner.update_banner(
                 time=self.utc,
                 lat=self.lat,
@@ -139,18 +142,19 @@ class NMEAHandler:
         except ValueError as err:
             self.__app.set_status(NMEAVALERROR.format(err), "red")
 
-    def _process_GGA(self, data: types.talker):
+    def _process_GGA(self, data: NMEAMessage):
         """
         Process GGA sentence - GPS Fix Data.
 
-        :param pynmea2.types.talker data: parsed GGA sentence
+        :param pynmeagps.NMEAMessage data: parsed GGA sentence
         """
 
         try:
-            self.utc = data.timestamp
-            self.sip = data.num_sats
-            (self.lat, self.lon) = nmea2latlon(data)
-            self.alt = data.altitude
+            self.utc = data.time
+            self.sip = data.numSV
+            self.lat = data.lat
+            self.lon = data.lon
+            self.alt = data.alt
             self.__app.frm_banner.update_banner(
                 time=self.utc, lat=self.lat, lon=self.lon, alt=self.alt, sip=self.sip
             )
@@ -158,28 +162,29 @@ class NMEAHandler:
             self.__app.frm_mapview.update_map(self.lat, self.lon, self.hacc)
 
             if (
-                self.__app.frm_settings.get_settings()["recordtrack"]
+                self.__app.frm_settings.record_track
                 and self.lat != ""
                 and self.lon != ""
             ):
-                tim = self.ts2utc(data.timestamp)
-                ele = 0 if self.alt == "None" else self.alt
+                tim = self.ts2utc(data.time)
+                ele = 0 if self.alt == "" else self.alt
                 self.__app.file_handler.add_trackpoint(
                     self.lat, self.lon, ele=ele, time=tim, sat=self.sip
                 )
         except ValueError as err:
             self.__app.set_status(NMEAVALERROR.format(err), "red")
 
-    def _process_GLL(self, data: types.talker):
+    def _process_GLL(self, data: NMEAMessage):
         """
         Process GLL sentence - GPS Lat Lon.
 
-        :param pynmea2.types.talker data: parsed GLL sentence
+        :param pynmeagps.NMEAMessage data: parsed GLL sentence
         """
 
         try:
-            self.utc = data.timestamp
-            (self.lat, self.lon) = nmea2latlon(data)
+            self.utc = data.time
+            self.lat = data.lat
+            self.lon = data.lon
             self.__app.frm_banner.update_banner(
                 time=self.utc, lat=self.lat, lon=self.lon
             )
@@ -189,19 +194,19 @@ class NMEAHandler:
         except ValueError as err:
             self.__app.set_status(NMEAVALERROR.format(err), "red")
 
-    def _process_GSA(self, data: types.talker):
+    def _process_GSA(self, data: NMEAMessage):
         """
         Process GSA sentence - GPS DOP (Dilution of Precision) and active satellites.
 
-        :param pynmea2.types.talker data: parsed GSA sentence
+        :param pynmeagps.NMEAMessage data: parsed GSA sentence
         """
 
-        self.pdop = float(data.pdop)
-        self.hdop = float(data.hdop)
-        self.vdop = float(data.vdop)
-        if data.mode_fix_type == "3":
+        self.pdop = data.PDOP
+        self.hdop = data.HDOP
+        self.vdop = data.VDOP
+        if data.navMode == 3:
             fix = "3D"
-        elif data.mode_fix_type == "2":
+        elif data.navMode == 2:
             fix = "2D"
         else:
             fix = "NO FIX"
@@ -212,7 +217,7 @@ class NMEAHandler:
             dop=self.pdop, hdop=self.hdop, vdop=self.vdop, fix=fix
         )
 
-    def _process_GSV(self, data: types.talker):
+    def _process_GSV(self, data: NMEAMessage):
         """
         Process GSV sentences - GPS Satellites in View
         These come in batches of 1-4 sentences, each containing the positions
@@ -220,10 +225,10 @@ class NMEAHandler:
         Modern receivers can send multiple batches corresponding to different
         NMEA assigned 'ID' ranges (GPS 1-32, SBAS 33-64, GLONASS 65-96 etc.)
 
-        :param pynmea2.types.talker data: parsed GSV sentence
+        :param pynmeagps.NMEAMessage data: parsed GSV sentence
         """
 
-        show_zero = self.__app.frm_settings.get_settings()["zerosignal"]
+        show_zero = self.__app.frm_settings.show_zero
         self.gsv_data = []
         gsv_dict = {}
         now = time()
@@ -237,48 +242,48 @@ class NMEAHandler:
             gnss = 0  # GPS, SBAS, QZSS
 
         try:
-            if data.sv_prn_num_1 != "":
-                svid = data.sv_prn_num_1
-                key = str(gnss) + "-" + svid
+            if data.svid_01 != "":
+                svid = data.svid_01
+                key = str(gnss) + "-" + str(svid)
                 gsv_dict[key] = (
                     gnss,
                     svid,
-                    data.elevation_deg_1,
-                    data.azimuth_1,
-                    data.snr_1,
+                    data.elv_01,
+                    data.az_01,
+                    str(data.cno_01),
                     now,
                 )
-            if data.sv_prn_num_2 != "":
-                svid = data.sv_prn_num_2
-                key = str(gnss) + "-" + svid
+            if data.svid_02 != "":
+                svid = data.svid_02
+                key = str(gnss) + "-" + str(svid)
                 gsv_dict[key] = (
                     gnss,
                     svid,
-                    data.elevation_deg_2,
-                    data.azimuth_2,
-                    data.snr_2,
+                    data.elv_02,
+                    data.az_02,
+                    str(data.cno_02),
                     now,
                 )
-            if data.sv_prn_num_3 != "":
-                svid = data.sv_prn_num_3
-                key = str(gnss) + "-" + svid
+            if data.svid_03 != "":
+                svid = data.svid_03
+                key = str(gnss) + "-" + str(svid)
                 gsv_dict[key] = (
                     gnss,
                     svid,
-                    data.elevation_deg_3,
-                    data.azimuth_3,
-                    data.snr_3,
+                    data.elv_03,
+                    data.az_03,
+                    str(data.cno_03),
                     now,
                 )
-            if data.sv_prn_num_4 != "":
-                svid = data.sv_prn_num_4
-                key = str(gnss) + "-" + svid
+            if data.svid_04 != "":
+                svid = data.svid_04
+                key = str(gnss) + "-" + str(svid)
                 gsv_dict[key] = (
                     gnss,
                     svid,
-                    data.elevation_deg_4,
-                    data.azimuth_4,
-                    data.snr_4,
+                    data.elv_04,
+                    data.az_04,
+                    str(data.cno_04),
                     now,
                 )
         except AttributeError:
@@ -298,31 +303,31 @@ class NMEAHandler:
         self.__app.frm_banner.update_banner(siv=len(self.gsv_data))
         self.__app.frm_graphview.update_graph(self.gsv_data, len(self.gsv_data))
 
-    def _process_VTG(self, data: types.talker):
+    def _process_VTG(self, data: NMEAMessage):
         """
         Process VTG sentence - GPS Vector track and Speed over the Ground.
 
-        :param pynmea2.types.talker data: parsed VTG sentence
+        :param pynmeagps.NMEAMessage data: parsed VTG sentence
         """
 
         try:
-            self.track = data.true_track
-            if data.spd_over_grnd_kmph is not None:
-                self.speed = kmph2ms(data.spd_over_grnd_kmph)  # convert to m/s
+            self.track = data.cogT
+            if data.sogk is not None:
+                self.speed = kmph2ms(data.sogk)  # convert to m/s
             self.__app.frm_banner.update_banner(speed=self.speed, track=self.track)
         except ValueError as err:
             self.__app.set_status(NMEAVALERROR.format(err), "red")
 
-    def _process_UBX00(self, data: types.ubx.UBX00):
+    def _process_UBX00(self, data: NMEAMessage):
         """
         Process UXB00 sentence - GPS Vector track and Speed over the Ground.
 
-        :param pynmea2.types.talker data: parsed UBX,00 sentence
+        :param pynmeagps.NMEAMessage data: parsed UBX,00 sentence
         """
 
         try:
-            self.hacc = float(data.h_acc)
-            self.vacc = float(data.v_acc)
+            self.hacc = data.hAcc
+            self.vacc = data.vAcc
             self.__app.frm_banner.update_banner(hacc=self.hacc, vacc=self.vacc)
 
         except ValueError as err:
