@@ -48,10 +48,6 @@ class SerialHandler:
         self._serial_thread = None
         self._file_thread = None
         self._connected = False
-        self._logpath = None
-        self._datalogging = False
-        self._recordtrack = False
-        self._port = None
         self._reading = False
 
     def __del__(self):
@@ -69,83 +65,76 @@ class SerialHandler:
         Open serial connection.
         """
 
-        settings = self.__app.frm_settings.get_settings()
-
-        self._port = settings["port"]
-        port_desc = settings["port_desc"]
-        baudrate = settings["baudrate"]
-        databits = settings["databits"]
-        stopbits = settings["stopbits"]
-        parity = settings["parity"]
-        xonxoff = settings["xonxoff"]
-        rtscts = settings["rtscts"]
-        timeout = settings["timeout"]
-        self._datalogging = settings["datalogging"]
-        self._recordtrack = settings["recordtrack"]
+        serial_settings = self.__app.frm_settings.serial_settings()
 
         try:
             self._serial_object = Serial(
-                self._port,
-                baudrate,
-                bytesize=databits,
-                stopbits=stopbits,
-                parity=parity,
-                xonxoff=xonxoff,
-                rtscts=rtscts,
-                timeout=timeout,
+                serial_settings.port,
+                serial_settings.bpsrate,
+                bytesize=serial_settings.databits,
+                stopbits=serial_settings.stopbits,
+                parity=serial_settings.parity,
+                xonxoff=serial_settings.xonxoff,
+                rtscts=serial_settings.rtscts,
+                timeout=serial_settings.timeout,
             )
             self._serial_buffer = BufferedReader(self._serial_object)
             self.__app.frm_banner.update_conn_status(CONNECTED)
             self.__app.set_connection(
-                f"{self._port}:{port_desc} @ {str(baudrate)}", "green"
+                (
+                    f"{serial_settings.port}:{serial_settings.port_desc} "
+                    + f"@ {str(serial_settings.bpsrate)}"
+                ),
+                "green",
             )
-            self.__app.frm_settings.set_controls(CONNECTED)
+            self.__app.frm_settings.enable_controls(CONNECTED)
             self._connected = True
             self.start_read_thread()
 
-            if self._datalogging:
+            if self.__app.frm_settings.datalogging:
                 self.__app.file_handler.open_logfile_output()
 
-            if self._recordtrack:
+            if self.__app.frm_settings.record_track:
                 self.__app.file_handler.open_trackfile()
 
         except (IOError, SerialException, SerialTimeoutException) as err:
             self._connected = False
             self.__app.set_connection(
-                f"{self._port}:{port_desc} @ {str(baudrate)}", "red"
+                (
+                    f"{serial_settings.port}:{serial_settings.port_desc} "
+                    + f"@ {str(serial_settings.baudrate)}"
+                ),
+                "red",
             )
             self.__app.set_status(SEROPENERROR.format(err), "red")
             self.__app.frm_banner.update_conn_status(DISCONNECTED)
-            self.__app.frm_settings.set_controls(DISCONNECTED)
+            self.__app.frm_settings.enable_controls(DISCONNECTED)
 
     def connect_file(self):
         """
         Open binary data file connection.
         """
 
-        settings = self.__app.frm_settings.get_settings()
-        self._datalogging = False
-        self._logpath = settings["logpath"]
-        self._recordtrack = settings["recordtrack"]
+        logpath = self.__app.frm_settings.logpath
 
         try:
-            self._serial_object = open(self._logpath, "rb")
+            self._serial_object = open(logpath, "rb")
             self._serial_buffer = BufferedReader(self._serial_object)
             self.__app.frm_banner.update_conn_status(CONNECTED_FILE)
-            self.__app.set_connection(f"{self._logpath}", "blue")
-            self.__app.frm_settings.set_controls(CONNECTED_FILE)
+            self.__app.set_connection(f"{logpath}", "blue")
+            self.__app.frm_settings.enable_controls(CONNECTED_FILE)
             self._connected = True
             self.start_readfile_thread()
 
-            if self._recordtrack:
+            if self.__app.frm_settings.record_track:
                 self.__app.file_handler.open_trackfile()
 
         except (IOError, SerialException, SerialTimeoutException) as err:
             self._connected = False
-            self.__app.set_connection(f"{self._logpath}", "red")
+            self.__app.set_connection(f"{logpath}", "red")
             self.__app.set_status(SEROPENERROR.format(err), "red")
             self.__app.frm_banner.update_conn_status(DISCONNECTED)
-            self.__app.frm_settings.set_controls(DISCONNECTED)
+            self.__app.frm_settings.enable_controls(DISCONNECTED)
 
     def disconnect(self):
         """
@@ -160,17 +149,17 @@ class SerialHandler:
                 self.__app.set_connection(NOTCONN, "red")
                 self.__app.set_status("", "blue")
 
-                if self._datalogging:
+                if self.__app.frm_settings.datalogging:
                     self.__app.file_handler.close_logfile()
 
-                if self._recordtrack:
+                if self.__app.frm_settings.record_track:
                     self.__app.file_handler.close_trackfile()
 
             except (SerialException, SerialTimeoutException):
                 pass
 
         self._connected = False
-        self.__app.frm_settings.set_controls(self._connected)
+        self.__app.frm_settings.enable_controls(self._connected)
 
     @property
     def port(self):
@@ -178,7 +167,7 @@ class SerialHandler:
         Getter for port
         """
 
-        return self._port
+        return self.__app.frm_settings.serial_settings().port
 
     @property
     def connected(self):
@@ -274,15 +263,18 @@ class SerialHandler:
         """
 
         try:
-            #         print("doing serial_handler._read_thread")
+            # print(f"DEBUG doing serial_handler._read_thread")
             while self._reading and self._serial_object:
-                #             print("doing serial_handler._read_thread while loop")
+                # print(f"DEBUG doing serial_handler._read_thread while loop")
                 if self._serial_object.in_waiting:
-                    #                 print(f"Bytes in buffer: {self._serial_object.in_waiting}")
-                    #                 print("doing serial_handler._read_thread in_waiting")
+                    # print(f"DEBUG Bytes in buffer: {self._serial_object.in_waiting}")
+                    # print(f"DEBUG doing serial_handler._read_thread in_waiting")
                     self.__master.event_generate("<<ubx_read>>")
         except SerialException as err:
             self.__app.set_status(f"Error in read thread {err}", "red")
+        # spurious errors as thread shuts down after serial disconnection
+        except (TypeError, OSError) as err:
+            pass
 
     def _readfile_thread(self):
         """
@@ -291,9 +283,9 @@ class SerialHandler:
         trigger data parsing and widget updates.
         """
 
-        #         print("doing serial_handler._readfile_thread")
+        # print(f"DEBUG doing serial_handler._readfile_thread")
         while self._reading and self._serial_object:
-            #             print("doing serial_handler._readfile_thread while loop")
+            # print(f"DEBUG doing serial_handler._readfile_thread while loop")
             self.__master.event_generate("<<ubx_readfile>>")
 
     def on_read(self, event):  # pylint: disable=unused-argument
@@ -303,7 +295,7 @@ class SerialHandler:
         :param event event: read event
         """
 
-        #         print("doing serial_handler.on_read")
+        # print(f"DEBUG doing serial_handler.on_read")
         if self._reading and self._serial_object is not None:
             try:
                 self._parse_data(self._serial_buffer)
@@ -317,7 +309,7 @@ class SerialHandler:
         :param event event: eof event
         """
 
-        #         print("doing serial_handler.on_eof")
+        # print(f"DEBUG doing serial_handler.on_eof")
         self.disconnect()
         self.__app.set_status(ENDOFFILE, "blue")
 
@@ -330,28 +322,25 @@ class SerialHandler:
         :param Serial ser: serial port
         """
 
-        #         print("doing serial_handler_parse_data")
+        # print(f"DEBUG doing serial_handler_parse_data")
         parsing = True
         raw_data = None
-        byte1 = ser.read(1)  # read first two bytes to determine protocol
+        byte1 = ser.read(1)  # read first byte to determine protocol
         if len(byte1) < 1:
             self.__master.event_generate("<<ubx_eof>>")
             return
 
         while parsing:
-            filt = self.__app.frm_settings.get_settings()["protocol"]
+            filt = self.__app.frm_settings.protocol
             byte2 = ser.read(1)
             if len(byte2) < 1:
                 self.__master.event_generate("<<ubx_eof>>")
                 return
             # if it's a UBX message (b'\b5\x62')
-            if (
-                byte1 == b"\xb5"
-                and byte2 == b"\x62"
-                and filt in (UBX_PROTOCOL, MIXED_PROTOCOL)
-            ):
-                #                 print(f"doing serial_handler._parse_data if ubx {ser.peek()}")
+            if byte1 == b"\xb5" and byte2 == b"\x62":
+                # print(f"DEBUG doing ubx serial_handler._parse_data if ubx {ser.peek()}")
                 byten = ser.read(4)
+                # print(f"DEBUG first byten = {byten}, len = {len(byten)}, need 4")
                 if len(byten) < 4:
                     self.__master.event_generate("<<ubx_eof>>")
                     parsing = False
@@ -361,6 +350,7 @@ class SerialHandler:
                 lenb = byten[2:4]
                 leni = int.from_bytes(lenb, "little", signed=False)
                 byten = ser.read(leni + 2)
+                # print(f"DEBUG second byten = {byten}, len = {len(byten)}, need {leni +2 }")
                 if len(byten) < leni + 2:
                     self.__master.event_generate("<<ubx_eof>>")
                     parsing = False
@@ -368,32 +358,23 @@ class SerialHandler:
                 plb = byten[0:leni]
                 cksum = byten[leni : leni + 2]
                 raw_data = ubt.UBX_HDR + clsid + msgid + lenb + plb + cksum
-                self.__app.ubx_handler.process_data(raw_data)
+                if filt in (UBX_PROTOCOL, MIXED_PROTOCOL):
+                    self.__app.ubx_handler.process_data(raw_data)
                 parsing = False
             # if it's an NMEA message ('$G' or '$P')
-            elif (
-                byte1 == b"\x24"
-                and byte2 in (b"\x47", b"\x50")
-                and filt
-                in (
-                    NMEA_PROTOCOL,
-                    MIXED_PROTOCOL,
-                )
-            ):
-                #                 print(f"doing serial_handler._parse_data if nmea {ser.peek()}")
-                try:
-                    raw_data = byte1 + byte2 + ser.readline()
-                    self.__app.nmea_handler.process_data(raw_data.decode("utf-8"))
-                except UnicodeDecodeError:
-                    continue
+            elif byte1 == b"\x24" and byte2 in (b"\x47", b"\x50"):
+                # print(f"DEBUG doing nmea serial_handler._parse_data if nmea {ser.peek()}")
+                raw_data = byte1 + byte2 + ser.readline()
+                if filt in (NMEA_PROTOCOL, MIXED_PROTOCOL):
+                    self.__app.nmea_handler.process_data(raw_data)
                 parsing = False
             # else drop it like it's hot
             else:
-                #                 print(f"dropping {ser.peek()}")
+                # print(f"DEBUG dropping {ser.peek()}")
                 parsing = False
 
         # if datalogging, write to log file
-        if self._datalogging and raw_data is not None:
+        if self.__app.frm_settings.datalogging and raw_data is not None:
             self.__app.file_handler.write_logfile(raw_data)
 
     def flush(self):
